/*
 *	htanaly.cpp
 */

/*	Copyright (C) 1999 Sebastian Biallas (sb@web-productions.de)
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License version 2 as
 *	published by the Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "htapp.h"
#include "htstring.h"
#include "htanaly.h"
#include "htidle.h"

/*
 *	CLASS ht_aviewer
 */


ht_aviewer::ht_aviewer(bounds *b, char *desc, htstream *file, int caps, analyser *_analy)
: ht_uviewer(b, desc, file, caps)
{
	analy=_analy;
	idle_count=0;
	last_active=1;
	register_idle_object(this);
}

ht_aviewer::~ht_aviewer()
{
	unregister_idle_object(this);
}

void ht_aviewer::idle()
{
	last_active=analy->active;
	analy->continueanalysis();
	if (last_active && !analy->active) {
		LOG("analyser finished after %d ops.", analy->ops_parsed);
		dirtyview();
		app->sendmsg(htm_draw, 0);
	}
	if (analy->active) {
		idle_count++;
		if (idle_count%50==0) {
			dirtyview();
			app->sendmsg(htm_draw, 0);
		}
	}
}

void ht_aviewer::handlemsg(htmsg *msg)
{
	if (msg->msg==htm_keypressed) {
		switch (msg->data_int) {
			case 'c':
				analy->continueanalysisat(cursor_eofs);
				clearmsg(msg);
				break;
		}
	}
	ht_uviewer::handlemsg(msg);
}

/*
 *	CLASS hts_analy
 */

hts_analy::hts_analy(htstream *file, analyser *A, ADDR Lowestaddress, ADDR Highestaddress)
:hts_sub(file)
{
	analy = A;
	lowestaddress=Lowestaddress;
	highestaddress=Highestaddress;
}

hts_analy::~hts_analy()
{
}

void hts_analy::end_line_address(dword *ofs, int *yofs)
{
	*ofs=highestaddress;
	*yofs=0;
}

int hts_analy::getline(char *line, dword ofs, int yofs)
{
	int len;
	analy->getformattedline(ofs, yofs+1, line, &len);
	return 1;
}

int	hts_analy::next_line_address(ADDR *ofs, int *yofs, int n)
{
	int len;
	int res=0;
	while (n--) {
		if (analy->getformattedline_length(*ofs, *yofs+2, &len)) {
			(*yofs)++;
		} else {
			analy->getformattedline_length(*ofs, *yofs+1, &len);
//     		if (!analy->validaddr(*ofs+len, scvalid)) return res;
			*yofs = 0;
			*ofs += len;
		}
		res++;
	}
	return res;
}

int	hts_analy::prev_line_address(ADDR *addr, int *line, int n)
{
	int len;

	ADDR addrbuf[1024];	/* FIXME: possible buffer overflow ! */
	int	linebuf[1024];	/* FIXME: possible buffer overflow ! */
	int	i=0;
	ADDR	l=n*15;
	if (l<64) l=64;

	/* look backwards */
//     printf("============= up ==============================\n");
//     printf("%d x from %lx line %d\n", n, *addr, *line);
	ADDR tmpaddr = (*addr) - l;
	ADDR tmpaddr2;
	int	tmpline;
	start_line_address(&tmpaddr2, &tmpline);
	if ((l > (*addr)) || (tmpaddr < tmpaddr2)) tmpaddr = tmpaddr2;
	ADDR	nextaddr = tmpaddr;
	while (1) {
		if (tmpaddr >= (*addr)) {
			if ((tmpline >= (*line)) || (tmpaddr > *addr)) break;
		}
		if (analy->getformattedline_length(tmpaddr, tmpline+1, &len)) {
			nextaddr += len;
			tmpline++;
		} else {
			tmpaddr = nextaddr;
			tmpline = 0;
			continue;
		}
//          printf("%2ld. %lx line %ld\n", i, tmpaddr, tmpline-1);
		addrbuf[i] = tmpaddr;
		linebuf[i] = tmpline-1;
		i++;
	}

	if (!i) return 0;

	if (i >= n) {
//     	printf("result1: %lx line %d (%d)\n", addrbuf[i-n], linebuf[i-n], n);
		(*addr) = addrbuf[i-n];
		(*line) = linebuf[i-n];
		return n;
	} else {
//     	printf("result2: %lx line %d\n (%d)\n", addrbuf[0], linebuf[0], i);
		(*addr) = addrbuf[0];
		(*line) = linebuf[0];
		return i;
	}
}

void	hts_analy::start_line_address(dword *ofs, int *yofs)
{
	*ofs=lowestaddress;
	*yofs=0;
}

int hts_analy::jump(ADDR tofs, ADDR *ofs, int *yofs)
{
	if (analy->validaddr(tofs, scvalid)) {
		*ofs=tofs;
		*yofs=0;
		return 1;
	}
	return 0;
}


