/* 
 *	The HT Editor
 *	htapp.cc
 *
 *	Copyright (C) 1999,2000 Stefan Weyergraf (stefan@weyergraf.de)
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License version 2 as
 *	published by the Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "analy.h"
#include "htapp.h"
#include "htdialog.h"
#include "hthelp.h"
#include "hthist.h"
#include "htkeyb.h"
#include "htpal.h"
#include "htstream.h"
#include "htstring.h"

#include "viewers.h"

#include <stdarg.h>
#include <string.h>

ht_app *app;
screendrawbuf *screen;

#define menu_palette_str				"menu"
#define menu_palette_colors			1
#define menu_coloridx_normal			0
#define menu_color_normal			pal[menu_coloridx_normal].value

#define keyline_palette_str			"keyline"
#define keyline_palette_colors          3
#define keyline_coloridx_digit		0
#define keyline_coloridx_active		1
#define keyline_coloridx_inactive		2
#define keyline_color_digit			pal[keyline_coloridx_digit].value
#define keyline_color_active			pal[keyline_coloridx_active].value
#define keyline_color_inactive		pal[keyline_coloridx_inactive].value

/*
 *	CLASS ht_menu
 */

ht_menu::ht_menu(bounds *b)
: ht_view(b, hto_ownbuffer, "menuline")
{
	setpalette(pal_grp->get(menu_palette_str));
}

ht_menu::~ht_menu()
{
}

void ht_menu::draw()
{
	buf->fill(size.x, size.y, size.w, size.h, menu_color_normal, ' ');
}

void ht_menu::handlemsg(htmsg *msg)
{
	ht_view::handlemsg(msg);
}

/*
 *	CLASS ht_keyline
 */

ht_keyline::ht_keyline(bounds *b)
: ht_view(b, 0, "keyline")
{
	setpalette(pal_grp->get(keyline_palette_str));
}

ht_keyline::~ht_keyline()
{
}

void ht_keyline::draw()
{
	buf->fill(size.x, size.y, size.w, size.h, keyline_color_inactive, ' ');
	int x=0;
	for (int i=1; i<=10; i++) {
		htmsg msg;
		msg.msg=htm_funcname;
		msg.data_int=i;
		app->sendmsg(&msg);
		printf(x, 0, keyline_color_digit, "%d", i%10);
		if (msg.msg==htm_funcname_return) {
			char *s=msg.data_str;
			if ((i==1) && (!s)) {
					  s="help";
			}
			if (s) {
				if (s[0]=='~') {
					printf(x+1, 0, keyline_color_inactive, s+1);
				} else {
					for (int j=0; j<size.w/10-1; j++) {
						printf(x+j+1, 0, keyline_color_active, " ");
					}
					printf(x+1, 0, keyline_color_active, s);
				}
			}
		}
		x+=size.w/10;
	}
}

void ht_keyline::handlemsg(htmsg *msg)
{
	ht_view::handlemsg(msg);
}

/*
 *	CLASS ht_app
 */

/*debug*/
//#define DRAW_TIMINGS
//#define NO_AVG
#define AVG_TIMINGS 10
int timings[AVG_TIMINGS];
int cur_timing=0, max_timing=0;
int h0;
/**/

ht_app::ht_app(bounds *_b)
: ht_group(_b, 0, "main application")
{
	focused=1;
	bounds b;
/* init timer */
	h0=new_timer();
/* create logwindow */
	getbounds(&b);
	b.y++;
	b.h-=2;

	ht_window *logwindow=new ht_window(&b, "log window", 1);
	b.x++;
	b.y++;
	b.w-=2;
	b.h-=2;
	logviewer=new ht_logviewer(&b, 128);
	logwindow->insert(logviewer);
	insert(logwindow);
/* create menu */
	getbounds(&b);
	b.h=1;
	menu=new ht_menu(&b);
	insert(menu);
/* create keyline */
	getbounds(&b);
	b.y=b.h-1;
	b.h=1;
	keyline=new ht_keyline(&b);
	insert(keyline);

	ht_view::setpalette(pal_grp->get(window_palette_str));
}

ht_app::~ht_app()
{
	delete_timer(h0);
}

int ht_app::load(htstream *f)
{
	if (!pal_grp->load(f)) return 0;
	return 1;
}

void ht_app::store(htstream *f)
{
	pal_grp->store(f);
}

void ht_app::draw()
{
/* show draw timings */
#ifdef DRAW_TIMING
	int xyz=get_timer_1024tick(h0);
	buf->printf(17, 1, 7, "cur: %d (%d msec)", xyz*1024, get_timer_msec(h0));
#ifndef NO_AVG
	if (cur_timing>=AVG_TIMINGS-1) cur_timing=0;
	timings[cur_timing++]=xyz;
	if (cur_timing>max_timing) max_timing=cur_timing;
	int avg=0;
	for (int i=0; i<max_timing; i++) avg+=timings[i];
	avg=avg/max_timing;
	buf->printf(57, 1, 7, "avg%d: %d", max_timing+1, avg*1024);
#endif
#endif
/* display flag if analyser(s) active */
	if (some_analyser_active) buf->printf(50, 1, window_color_body, "ANALY(%d)", num_ops_parsed);
}

void ht_app::delete_window(int index)
{
}

int ht_app::findfreewinnumber()
{
	int i=1;
	int free;
	while (1) {
		ht_view *w=first;
		free=1;
		while (w) {
			if (i==w->getnumber()) {
				free=0;
				break;
			}
			w=w->next;
		}
		if (free) return i;
		i++;
	}
	return -1;
}

ht_window *ht_app::findwindowbynumber(int index)
{
	ht_view *w=first;
	while (w) {
		if (index==w->getnumber()) return (ht_window*)w;
		w=w->next;
	}
	return 0;
}

void ht_app::focus_window(int index)
{
	ht_window *w=findwindowbynumber(index);
	if (w) focus(w);
}

void ht_app::handlemsg(htmsg *msg)
{
	if (msg->msg==htm_draw) {
		start_timer(h0);
		if (msg->type==htmt_broadcast) {
			ht_view *v=first;
			while (v) {
				v->handlemsg(msg);
				v=v->next;
			}
		} else {
			current->handlemsg(msg);
		}
		stop_timer(h0);
		draw();
		screen->show();
	} else {
		ht_group::handlemsg(msg);
	}
	if (msg->msg==htm_keypressed) {
		int i=0;
		switch (msg->data_int) {
			case K_F1:
				popup_help_dialog("htinfo", "Top");
				clearmsg(msg);
				break;
			case K_Alt_9: i++;
			case K_Alt_8: i++;
			case K_Alt_7: i++;
			case K_Alt_6: i++;
			case K_Alt_5: i++;
			case K_Alt_4: i++;
			case K_Alt_3: i++;
			case K_Alt_2: i++;
			case K_Alt_1: i++;
				focus_window(i);
				clearmsg(msg);
				break;
			case K_Alt_0:
				popup_window_list();
				clearmsg(msg);
				break;
			case K_Return:
				popup_view_list();
				clearmsg(msg);
				break;
			case K_Control_K:
				popup_color_list();
				clearmsg(msg);
				break;
			case K_Space: {
/*				bounds b, c, d, e;
				getbounds(&b);
				b.x=b.w/4;
				b.y=b.h/4;
				b.w/=2;
				b.h/=2;
				ht_dialog *dialog=new ht_dialog(&b, "ht-vision demo");
				c.x=b.x+2;
				c.y=b.y+2;
				c.w=20;
				c.h=2;
				ht_inputfield *inputfield=new ht_inputfield(&c, "hallihallo0123456789-0");
				dialog->insert(inputfield);

				d.x=b.x+2;
				d.y=b.y+5;
				d.w=20;
				d.h=3;
				ht_listbox *listbox=new ht_listbox(&d);
				listbox->insertstring("string001");
				listbox->insertstring("sdfsdfsdfsdfg");
				listbox->insertstring("aset4356");
				listbox->insertstring("3--aset4356");
				listbox->insertstring("=///aset4356");
				dialog->insert(listbox);

				e.x=b.x+2;
				e.y=b.y+10;
				e.w=8;
				e.h=1;
				ht_button *button=new ht_button(&e, "Ok", 0);
				dialog->insert(button);
				execute_dialog(dialog);
				delete dialog;
				clearmsg(msg);
				break;*/
			}
		}
	}
}

int ht_app::new_window(char *filename)
{
/*	fprintf(stderr, "file: %s\n", filename);*/
	bounds b;
	getbounds(&b);
	b.y++;
	b.h-=2;
	LOG("loading file \"%s\"...", filename);
	htrwstream *file=new htrwstream(filename);
	if (!file->file) {
		return 0;
	}
	int winnumber=findfreewinnumber();
	ht_window *window=new ht_window(&b, filename, winnumber);
	LOG("%s: allocated window %d.", file->getname(), winnumber);
	b.x++;
	b.y++;
	b.w-=2;
	b.h-=2;
	ht_xgroup *xgroup=new ht_xgroup(&b, hto_selectable, filename);

	viewer_int **i=viewer_ints;
	while (*i) {
		if (((*i)->init) && ((*i)->detect) && ((*i)->detect(file))) {
			ht_view *v=(*i)->init(&b, file, window);
			v->sendmsg(htm_complete, 0);
			xgroup->insert(v);
		}
		i++;
	}
	window->insert(xgroup);
	LOG("%s: ready.", filename);
	insert(window);
	focus(window);
	return 1;
}

int ht_app::logf(const char *format, ...)
{
	char buf[1024];	/* FIXME: possible buffer overflow ! */
	va_list arg;
	va_start(arg, format);
	int r=vsprintf(buf, format, arg);
	va_end(arg);
	logviewer->insertline(buf);
	sendmsg(htm_draw, 0);
	return r;
}

void ht_app::popup_color_list()
{
	bounds b;
	getbounds(&b);
	b.x=b.w/6;
	b.y=b.h/6;
	b.w=2*b.w/3;
	b.h=2*b.h/3;
	ht_dialog *dialog=new ht_color_manager(&b, pal_grp);
	if (execute_dialog(dialog)) {
		ht_color_manager_data data;
		dialog->getdata(&data);
		pal_grp->paste(data.pal_grp);
	}
	delete dialog;
}

void ht_app::popup_view_list()
{
	bounds b, c;
	getbounds(&b);
	b.x=b.w/4;
	b.y=b.h/4;
	b.w/=2;
	b.h/=2;
	ht_dialog *dialog=new ht_dialog(&b, "views");

/* create listbox */
	c=b;
	c.x++;
	c.y++;
	c.w-=2;
	c.h-=2;
	ht_listbox_button *listbox=new ht_listbox_button(&c, button_ok);
/* insert all browsable views */
	ht_collection *structure=new ht_collection();
	int index=0;
	popup_view_list_dump(current, listbox, structure, 0, &index, getcurrentchild());
/* and seek to the currently selected one */
	listbox->seek(index);

	dialog->insert(listbox);
	dialog->setpalette(pal_grp->get(dialog_special_palette_str));

	if (execute_dialog(dialog)) {
		ht_listbox_data data;
		listbox->getdata(&data);
		ht_data_ptr *p=(ht_data_ptr*)structure->get(data.index);
		focus((ht_view*)p->data);
	}
	delete dialog;
}

void ht_app::popup_view_list_dump(ht_view *view, ht_listbox *listbox, ht_collection *structure, int depth, int *currenti, ht_view *currentv)
{
	char str[256];
	char *s=str;
	if (!(view->options & hto_browsable)) {
		depth--;
	}
	for (int i=0; i<depth; i++) s+=sprintf(s, "  ");
	if (!view) return;
	int c=view->childcount();
	for (int i=0; i<c; i++) {
		ht_view *v=view->getfirstchild();
		while (v) {
			if (v->browse_idx==i) break;
			v=v->next;
		}
		if (!v) return;
		if (v->options & hto_browsable) {
			sprintf(s, "- %s", v->desc);
			structure->add(new ht_data_ptr(v));
			if (v==currentv)
				*currenti=structure->count()-1;
			listbox->insertstring(str);
		}
		popup_view_list_dump(v, listbox, structure, depth+1, currenti, currentv);
	}
}

void ht_app::popup_window_list()
{
	bounds b, c;
	getbounds(&b);
	b.x=b.w/4;
	b.y=b.h/4;
	b.w/=2;
	b.h/=2;
	ht_dialog *dialog=new ht_dialog(&b, "windows");

/* create listbox */
	c=b;
	c.x++;
	c.y++;
	c.w-=2;
	c.h-=2;
	ht_listbox_button *listbox=new ht_listbox_button(&c, button_ok);

	int view_count=0;
	ht_view *v=first;
	while (v) {
		view_count++;
		v=v->next;
	}

	for (int i=0; i<view_count; i++) {
		ht_view *v=first;
		while (v) {
			if ((v->getnumber()==i) && (v->options & hto_browsable)) {
				char l[1024];	/* FIXME: possible buffer overflow ! */
				sprintf(l, "%2d %s", i, v->desc);
				listbox->insertstring(l);
			}
			v=v->next;
		}
	}
	listbox->seek(current->getnumber()-1);
	dialog->insert(listbox);
	dialog->setpalette(pal_grp->get(dialog_special_palette_str));

	if (execute_dialog(dialog)) {
		ht_listbox_data data;
		listbox->getdata(&data);
		focus_window(data.index+1);
	}
	delete dialog;
}

/*
 *	INIT
 */

int init_app()
{
	palette *menu_palette=pal_grp->create(menu_palette_str, menu_palette_colors);
	pal_grp->set_entry(menu_palette, menu_coloridx_normal, "normal", COLOR(C_WHITE, C_BLACK));

	palette *keyline_palette=pal_grp->create(keyline_palette_str, keyline_palette_colors);
	pal_grp->set_entry(keyline_palette, keyline_coloridx_digit, "digit", COLOR(C_WHITE, C_BLACK));
	pal_grp->set_entry(keyline_palette, keyline_coloridx_active, "active", COLOR(C_BLACK, C_CYAN));
	pal_grp->set_entry(keyline_palette, keyline_coloridx_inactive, "inactive", COLOR(C_WHITE, C_CYAN));

	bounds b;
	screen=new screendrawbuf();

	b.x=0;
	b.y=0;
	b.w=screen->size.w;
	b.h=screen->size.h;
	app=new ht_app(&b);

	return 1;
}

/*
 *	DONE
 */

void done_app()
{
	delete app;
	delete screen;
}

void backref()
{
	vstate *vs=vs_history->pop();
	if (vs) {
		app->focus(vs->view);
		vs->view->sendmsg(htm_vs_restore, (int)vs->data);
		delete vs;
	}
}

