/* 
 *	The HT Editor
 *	htdebug.h
 *
 *	Copyright (C) 1999,2000 Stefan Weyergraf (stefan@weyergraf.de)
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License version 2 as
 *	published by the Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __HTDEBUG_H__
#define __HTDEBUG_H__

#include <stdio.h>

/****************************************************************************/
#define HTDEBUG
/****************************************************************************/

void ht_assert_failed(char *file, int line, char *assertion);
void ht_warn(char *file, int line, char *format,...);
void ht_error(char *file, int line, char *format,...);

int new_timer();
void start_timer(int handle);
void stop_timer(int handle);
void delete_timer(int h);

int get_timer_sec(int handle);
int get_timer_msec(int handle);
int get_timer_tick(int handle);

#define WARN(a...) ht_warn(__FILE__, __LINE__, a)
#define ERROR(a...) ht_error(__FILE__, __LINE__, a)

#ifdef HTDEBUG
	#define assert(a) if (!(a)) ht_assert_failed(__FILE__, __LINE__, (#a));
#else
	#define assert(a) ((void)0)
#endif

#define abstract() ERROR("abstract method called\n")

#endif /* !__HTDEBUG_H__ */
