/* 
 *	The HT Editor
 *	htdialog.h
 *
 *	Copyright (C) 1999,2000 Stefan Weyergraf (stefan@weyergraf.de)
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License version 2 as
 *	published by the Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __HTDIALOG_H__
#define __HTDIALOG_H__

#include "htdebug.h"
#include "htobj.h"

/*
 *	Palette entries
 */

#define dialog_cyan_palette_str				"dialog (cyan)"
#define dialog_black_palette_str				"dialog (black)"
#define dialog_gray_palette_str				"dialog (gray)"

#define dialog_special_palette_str				"dialog (special)"

#define dialog_palette_str					dialog_black_palette_str
#define dialog_palette_colors           		14

#define dialog_coloridx_body					0
#define dialog_coloridx_frame					1
#define dialog_coloridx_killer				2
#define dialog_coloridx_scrollbar				3

#define dialog_coloridx_input_focused			4
#define dialog_coloridx_input_unfocused			5
#define dialog_coloridx_input_clip 			6

#define dialog_coloridx_button_focused			7
#define dialog_coloridx_button_unfocused		8
#define dialog_coloridx_button_shadow 			9

#define dialog_coloridx_list_focused_selected	10
#define dialog_coloridx_list_focused_unselected	11
#define dialog_coloridx_list_unfocused_selected	12
#define dialog_coloridx_list_unfocused_unselected	13

#define dialog_color_body					pal[dialog_coloridx_body].value
#define dialog_color_frame					pal[dialog_coloridx_frame].value
#define dialog_color_killer					pal[dialog_coloridx_killer].value
#define dialog_color_scrollbar				pal[dialog_coloridx_scrollbar].value

#define dialog_color_input_focused				pal[dialog_coloridx_input_focused].value
#define dialog_color_input_unfocused			pal[dialog_coloridx_input_unfocused].value
#define dialog_color_input_clip				pal[dialog_coloridx_input_clip].value

#define dialog_color_button_focused			pal[dialog_coloridx_button_focused].value
#define dialog_color_button_unfocused			pal[dialog_coloridx_button_unfocused].value
#define dialog_color_button_shadow				pal[dialog_coloridx_button_shadow].value

#define dialog_color_list_focused_selected		pal[dialog_coloridx_list_focused_selected].value
#define dialog_color_list_focused_unselected		pal[dialog_coloridx_list_focused_unselected].value
#define dialog_color_list_unfocused_selected		pal[dialog_coloridx_list_unfocused_selected].value
#define dialog_color_list_unfocused_unselected	pal[dialog_coloridx_list_unfocused_unselected].value

/*
 *	CLASS ht_dialogframe
 */

class ht_dialogframe: public ht_view {
public:
			ht_dialogframe(bounds *b, char *desc);
	virtual 	~ht_dialogframe();
/* overwritten */
STUB	virtual	void draw();
STUB	virtual	void handlemsg(htmsg *msg);
};

/*
 *	CLASS ht_dialog
 */

#define ds_normal			0
#define ds_term_ok			1
#define ds_term_cancel		2

#define button_cancel		0
#define button_ok			1
#define button_yes            button_ok
#define button_no			2

class ht_dialog: public ht_group {
private:
	int state;
	ht_dialogframe *frame;
public:
	ht_dialog(bounds *b, char *desc);
	~ht_dialog();
/* overridden */
	virtual 	void draw();
	virtual	void handlemsg(htmsg *msg);
/* new */
	virtual	int getstate();
	virtual	void setstate(int state);
};

/*
 *	CLASS ht_inputfield
 */

struct ht_inputfield_data {
	char *text;
};

class ht_inputfield: public ht_view {
protected:
	char *text;
	char *curchar;
	int insert;
	int maxstrlen;
	int ofs;
public:
	ht_inputfield(bounds *b, char *deftext, int maxstrlen);
	virtual ~ht_inputfield();
/* overwritten */
	virtual 	void draw();
	virtual	void getdata(void *buf);
	virtual	int getdatasize();
	virtual	void handlemsg(htmsg *msg);
/* new */
			void setchar(char a, int insert);
};

/*
 *	CLASS ht_inputfield_button
 */

struct ht_inputfield_button_data {
	char *text;
	int pressed;
};

class ht_inputfield_button: public ht_inputfield {
private:
	int value;
	int pressed;
public:
	ht_inputfield_button(bounds *b, char *deftext, int _maxstrlen, int value);
	virtual ~ht_inputfield_button();
/* overwritten */
	virtual	void getdata(void *buf);
	virtual	int getdatasize();
	virtual	void handlemsg(htmsg *msg);
};

/*
 *	CLASS ht_button
 */

struct ht_button_data {
	int pressed;
};

class ht_button: public ht_view {
protected:
	int value;
	int pressed;
	char *text;
public:
	ht_button(bounds *b, char *text, int value);
	virtual ~ht_button();
/* overwritten */
	virtual 	void draw();
	virtual	void getdata(void *buf);
	virtual	int getdatasize();
	virtual	void handlemsg(htmsg *msg);
};

/*
 *	CLASS ht_listbox_body
 */

struct ht_listbox_data {
	ht_string_collection *list;
	int index;
};

class ht_listbox_body: public ht_view {
protected:
	ht_string_collection *list;
	int pos, cursor;
	int vis_h_size;
	ht_scrollbar *scrollbar;
public:
	ht_listbox_body(bounds *b, ht_scrollbar *scrollbar);
	virtual ~ht_listbox_body();
/* overwritten */
	virtual	void draw();
	virtual	void getdata(void *buf);
	virtual	int getdatasize();
	virtual	void handlemsg(htmsg *msg);
/* new */
	virtual	void insertstring(char *str);
			int seek(int index);
};

/*
 *	CLASS ht_listbox
 */

class ht_listbox: public ht_group {
protected:
	ht_listbox_body *body;
	ht_scrollbar *scrollbar;
public:
	ht_listbox(bounds *b);
	~ht_listbox();
/* overwritten */
	virtual	void getdata(void *buf);
	virtual	int getdatasize();
	virtual	void setpalette(palette *palette);
/* new */
	void insertstring(char *str);
	int seek(int index);
};

/*
 *	CLASS ht_listbox_button
 */

class ht_listbox_button: public ht_listbox {
private:
	int value;
public:
	ht_listbox_button(bounds *b, int value);
	virtual ~ht_listbox_button();
/* overwritten */
	virtual	void handlemsg(htmsg *msg);
};

/*
 *	CLASS ht_listpopup_body
 */

#define ht_listpopup_data ht_listbox_data

class ht_listpopup_body: public ht_listbox_body {
protected:
	int expanded;
public:
	ht_listpopup_body(bounds *b, ht_scrollbar *scrollbar);
	virtual ~ht_listpopup_body();
/* overwritten */
			void collapse();
	virtual	void draw();
			void expand();
	virtual	void handlemsg(htmsg *msg);
	virtual	void insertstring(char *str);
	virtual	void releasefocus();
};

/*
 *	CLASS ht_listpopup
 */

class ht_listpopup: public ht_group {
protected:
	ht_listbox_body *body;
	ht_scrollbar *scrollbar;
public:
	ht_listpopup(bounds *b);
	~ht_listpopup();
/* overwritten */
	virtual	void getdata(void *buf);
	virtual	int getdatasize();
	virtual	void setpalette(palette *palette);
/* new */
	void insertstring(char *str);
};

/*
 *	CLASS ht_statictext
 */

struct ht_statictext_data {
	char *text;
};

class ht_statictext: public ht_view {
private:
	char *text;
public:
	ht_statictext(bounds *b, char *text);
	virtual ~ht_statictext();
/* overwritten */
	virtual 	void draw();
	virtual	void getdata(void *buf);
	virtual	int getdatasize();
};

/*
 *	CLASS ht_color_block
 */

struct ht_color_block_data {
	int color;
};

class ht_color_block: public ht_view {
protected:
	int color;
	int colors;
public:
	ht_color_block(bounds *b, int light_colors, int selected);
	~ht_color_block();
/* overwritten */
	virtual	void draw();
	virtual	void getdata(void *buf);
	virtual	int getdatasize();
	virtual	void handlemsg(htmsg *msg);
/* new */
	virtual	void set_color(int i);
};

/*
 *	CLASS ht_listbox_ptr
 */

class ht_listbox_ptr: public ht_data {
public:
	ht_listbox *listbox;
	ht_listbox_ptr(ht_listbox *listbox);
	~ht_listbox_ptr();
};

/*
 *	CLASS ht_color_manager
 */

struct ht_color_manager_data {
	palette_group *pal_grp;
};

class ht_color_manager: public ht_dialog {
protected:
	ht_listbox *group;
	ht_xgroup *items;
	ht_collection *group_to_items;
	ht_color_block *fg_colors;
	ht_color_block *bg_colors;
	palette_group *orig_pal_grp;
	palette_group *pal_grp;
public:
	ht_color_manager(bounds *b, palette_group *pal);
	~ht_color_manager();
/* overridden */
	virtual	void getdata(void *buf);
	virtual	int getdatasize();
	virtual	void handlemsg(htmsg *msg);
};

/*
 *	misc
 */

#define msgbox_ok		1
#define msgbox_cancel	2
#define msgbox_yes		4
#define msgbox_no		8

int execute_dialog(ht_dialog *dialog);
int msgbox(int buttonmask, char *title, char *format, ...);
int warnbox(char *format, ...);
int errorbox(char *format, ...);

/*
 *	INIT
 */

int init_dialog();

/*
 *	DONE
 */

int done_dialog();

#endif /* !__HTDIALOG_H__ */

