/* 
 *	The HT Editor
 *	htpedimp.cc
 *
 *	Copyright (C) 1999,2000 Stefan Weyergraf (stefan@weyergraf.de)
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License version 2 as
 *	published by the Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "htapp.h"
#include "htnewexe.h"
#include "htpe.h"
#include "htpedimp.h"
#include "htstream.h"
#include "htstring.h"
#include "viewers.h"

#include <stdlib.h>

int htpedelayimports_detect(htstream *file)
{
	int h=get_newexe_header_ofs(file);
	dword pe=0;
	file->seek(h);
	file->read(&pe, 2);
	if (pe==PE_MAGIC) {
		int e, f;
		file->seek(h+0x78+13*8);
		file->read(&e, 4);
		file->read(&f, 4);
		return e|f;
	}
	return 0;
}

ht_view *htpedelayimports_init(bounds *b, htstream *file, ht_group *group)
{
	ht_pe_shared_data *pe_shared=(ht_pe_shared_data *)((ht_pe*)group)->shared_data;
	dword base=pe_shared->pe32header.image_base;
	int h=get_newexe_header_ofs(file);
	char iline[1024];	/* FIXME: possible buffer overflow */

	int dll_count=0;
	int function_count=0;

	ht_group *g;
	bounds c;
	ht_winstatictext *head;

	c=*b;
	g=new ht_group(&c, 0, "pe/delay-imports-g");

	c.y++;
	c.h--;
	ht_uviewer *v=new ht_pe_dimport_viewer(&c, DESC_PE_DIMPORTS, file, 0);
	hts_mask *m=new hts_mask(file, h);

	c.y--;
	c.h=1;
/* get delay import directory offset */
	/* 1. get import directory rva */
	dword iofs, irva=pe_shared->pe32header.directory[PE_DIRECTORY_ENTRY_DELAY_IMPORT].address;
	/* 2. transform it into an offset */
	if (!pe_rva_to_ofs(&pe_shared->sections, irva, &iofs)) goto pe_read_error;
	LOG("%s: PE: reading delay-import directory at offset %08x, rva %08x, size %08x...", file->getname(), iofs, irva, pe_shared->pe32header.directory[PE_DIRECTORY_ENTRY_DELAY_IMPORT].size);
/*** read delay import directory ***/
	PE_DELAY_IMPORT_DESCRIPTOR dimport;
	int dll_index;
	dll_index=0;
	dword f;
	f=0;

	while (1) {
		file->seek(iofs);
		file->read(&dimport, sizeof dimport);
		if (!dimport.name) break;
		dword dllname_ofs;
		if (!pe_rva_to_ofs(&pe_shared->sections, dimport.name-base, &dllname_ofs)) goto pe_read_error;
		file->seek(dllname_ofs);
		char *dllname=getstrz(file);
		ht_pe_dimport_library *lib=new ht_pe_dimport_library(dllname);
		pe_shared->dimports.libs.add(lib);
		dll_count++;

		dword ntofs=dimport.delay_int-base, nva, nofs;
		dword atofs=dimport.delay_iat-base, ava;
		while (1) {
			function_count++;
			ht_pe_dimport_function *func;
			file->seek(ntofs);
			file->read(&nva, 4);
			if (!nva) break;
			file->seek(atofs);
			file->read(&ava, 4);
			if (nva & 0x80000000) {
				/* import by ordinal */
				char ord[5];
				sprintf(ord, "%04lx", nva&0xffff);
				char *im=iline;
				im+=sprintf(iline, "%-14s %08lx ", dllname, ava);
				mktag_ref(im, f, tag_coloridx_sel_unselected, ord, ava);
//				sprintf(iline, "%-14s %08lx "_SEL("%08lx", "04")"%04lx", dllname, nva, f, nva&0xffff);
				m->add_mask(iline);
				func=new ht_pe_dimport_function(dll_index, ava, nva&0xffff);
			} else {
				/* import by name */
				if (!pe_rva_to_ofs(&pe_shared->sections, nva-base, &nofs)) goto pe_read_error;
				int hint=0;
				file->seek(nofs);
				file->read(&hint, 2);
				char *name=getstrz(file);
/* FIXME: what about the hint ? */
				char *im=iline;
				im+=sprintf(iline, "%-14s %08lx ", dllname, nva);
				mktag_ref(im, f, tag_coloridx_sel_unselected, name, ava);
//				sprintf(iline, "%-14s %08lx "_SEL("%08lx", "%02lx")"%s", dllname, nva, f, strlen(name), name);
				m->add_mask(iline);
				func=new ht_pe_dimport_function(dll_index, ava, name, hint);
				delete name;
			}
			pe_shared->dimports.funcs.add(func);
			ntofs+=4;
			atofs+=4;
			f++;
		}

		delete dllname;
		iofs+=sizeof dimport;
		dll_index++;
	}

	if (!function_count) m->add_mask(_SEL("ffffffff", "1d")"<no delay-imported functions>");
	v->insertsub(m);

	sprintf(iline, "Library        FT_VA    Name/Ordinal    %d delay-imports from %d libraries", function_count, dll_count);
	head=new ht_winstatictext(&c, iline);

	g->insert(head);
	g->insert(v);

	return g;
pe_read_error:
	assert(0);
	return 0;
}

viewer_int htpedelayimports_int = {
	htpedelayimports_detect,
	htpedelayimports_init
};

/*
 *	CLASS ht_pe_dimport_viewer
 */

ht_pe_dimport_viewer::ht_pe_dimport_viewer(bounds *b, char *desc, htstream *file, int caps)
: ht_uviewer(b, desc, file, caps)
{
}

ht_pe_dimport_viewer::~ht_pe_dimport_viewer()
{
}

void ht_pe_dimport_viewer::ref(dword id)
{
	ht_group *g=group->group;
	ht_viewer *v=(ht_viewer*)g->getbydesc(DESC_PE_IMAGE);
	if (v) {
		v->jump(id);
		group->group->focus(v);
	}
}

/*
 *	CLASS ht_pe_dimport_library
 */

ht_pe_dimport_library::ht_pe_dimport_library()
{
}

ht_pe_dimport_library::ht_pe_dimport_library(char *name)
{
	this->name=htstrdup(name);
}

ht_pe_dimport_library::~ht_pe_dimport_library()
{
}

/*
 *	CLASS ht_pe_dimport_function
 */

ht_pe_dimport_function::ht_pe_dimport_function()
{
	dll=0;
	name.name=0;
	name.hint=0;
	ordinal=0;
	function_address=0;
}

ht_pe_dimport_function::ht_pe_dimport_function(int dll, int function_address, int ordinal)
{
	this->dll=dll;
	this->ordinal=ordinal;
	this->function_address=function_address;
	byname=0;
}

ht_pe_dimport_function::ht_pe_dimport_function(int dll, int function_address, char *name, int hint)
{
	this->dll=dll;
	this->name.name=htstrdup(name);
	this->name.hint=hint;
	this->function_address=function_address;
	byname=1;
}

ht_pe_dimport_function::~ht_pe_dimport_function()
{
	if ((byname) && (name.name)) delete name.name;
}
