/* 
 *	The HT Editor
 *	htpeexp.cc
 *
 *	Copyright (C) 1999,2000 Stefan Weyergraf (stefan@weyergraf.de)
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License version 2 as
 *	published by the Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "htapp.h"
#include "htdebug.h"
#include "htnewexe.h"
#include "htpe.h"
#include "htpeexp.h"
#include "htstream.h"
#include "htstring.h"
#include "viewers.h"

#include <stdlib.h>
#include <string.h>

int htpeexports_detect(htstream *file)
{
	int h=get_newexe_header_ofs(file);
	dword pe=0;
	file->seek(h);
	file->read(&pe, 2);
	if (pe==PE_MAGIC) {
		int e, f;
		file->seek(h+0x78+0*8);
		file->read(&e, 4);
		file->read(&f, 4);
		return e|f;
	}
	return 0;
}

ht_view *htpeexports_init(bounds *b, htstream *file, ht_group *group)
{
	int h0=new_timer();
	start_timer(h0);
	ht_group *g;

	ht_pe_shared_data *pe_shared=(ht_pe_shared_data *)((ht_pe*)group)->shared_data;
	int h=get_newexe_header_ofs(file);
	char eline[1024];   /* FIXME: possible buffer overflow */
	char *elinep;
	dword *efunct=0, *enamet=0;
	word *eordt=0;
	bounds c;
	ht_winstatictext *head;
	char *filename=file->getname();

	c=*b;
	g=new ht_group(&c, 0, "pe/exports-g");

	c.y++;
	c.h--;
	ht_uviewer *v=new ht_pe_export_viewer(&c, DESC_PE_EXPORTS, file, 0);
	hts_mask *m=new hts_mask(file, h);

	c.y--;
	c.h=1;

/* get export directory offset */
	/* 1. get export directory rva */
	dword eofs, erva=pe_shared->pe32header.directory[PE_DIRECTORY_ENTRY_EXPORT].address, esize=pe_shared->pe32header.directory[PE_DIRECTORY_ENTRY_EXPORT].size;
	/* 2. transform it into an offset */
	if (!pe_rva_to_ofs(&pe_shared->sections, erva, &eofs)) goto pe_read_error;
	LOG("%s: PE: reading export directory at offset %08x, rva %08x, size %08x...", filename, eofs, erva, esize);
/* make a memfile out of this section */
	htstream *efile;
	char *esectionbuf;
	efile=new htmemstream(eofs, esize);
	esectionbuf=(char*)malloc(esize);
	file->seek(eofs);
	file->read(esectionbuf, esize);
	efile->write(esectionbuf, esize);
	file=efile;
	delete esectionbuf;
/* read export directory header */
	PE_EXPORT_DIRECTORY edir;
	file->seek(eofs);
	file->read(&edir, sizeof edir);
/* get export filename */
	dword ename_ofs;
	char *ename;
	if (!pe_rva_to_ofs(&pe_shared->sections, edir.name_address, &ename_ofs)) goto pe_read_error;
	file->seek(ename_ofs);
	ename=getstrz(file);
/* ...and add it to the mask */
	sprintf(eline, "Ord  VA       Name      %s exports %ld functions", ename, edir.function_count);
	head=new ht_winstatictext(&c, eline);
	delete ename;
/* read in function entrypoint table */
	dword efunct_ofs;
	efunct=(dword*)malloc(edir.function_count*sizeof *efunct);
	if (!pe_rva_to_ofs(&pe_shared->sections, edir.function_table_address, &efunct_ofs)) goto pe_read_error;
	file->seek(efunct_ofs);
	file->read(efunct, edir.function_count*sizeof *efunct);
/* read in name address table */
	dword enamet_ofs;
	enamet=(dword*)malloc(edir.name_count*sizeof *efunct);
	if (!pe_rva_to_ofs(&pe_shared->sections, edir.name_table_address, &enamet_ofs)) goto pe_read_error;
	file->seek(enamet_ofs);
	file->read(enamet, edir.name_count*sizeof *efunct);
/* read in ordinal table */
	dword eordt_ofs;
	eordt=(word*)malloc(edir.name_count*sizeof *efunct);
	if (!pe_rva_to_ofs(&pe_shared->sections, edir.ordinal_table_address, &eordt_ofs)) goto pe_read_error;
	file->seek(eordt_ofs);
	file->read(eordt, edir.name_count*sizeof *efunct);

	int *lord;
	lord=(int*)malloc(sizeof *lord*edir.function_count);
	memset(lord, 0, sizeof *lord*edir.function_count);
	for (dword i=0; i<edir.name_count; i++) {
		*(lord+*(eordt+i))=1;
	}

/* exports by ordinal */
	for (dword i=0; i<edir.function_count; i++) {
		if (*(lord+i)) continue;

		dword f=*(efunct+i)+pe_shared->pe32header.image_base;

//		sprintf(eline, "%04lx %08lx "_SEL("%08lx", "0e")"<ordinal only>", i+edir.ordinal_base, f, i);

		elinep=eline;
		elinep+=sprintf(elinep, "%04lx %08lx ", i+edir.ordinal_base, f);
		mktag_ref(elinep, i, tag_coloridx_sel_unselected, "<ordinal only>", 0xffffffff);

		ht_pe_export_function *efd=new ht_pe_export_function(f, i+edir.ordinal_base);
		pe_shared->exports.funcs.add(efd);
		m->add_mask(eline);
	}
	delete lord;
/* exports by name */
	for (dword i=0; i<edir.name_count; i++) {
		dword o=*(eordt+i);
		dword f=*(efunct+o)+pe_shared->pe32header.image_base;
		dword en;
		if (!pe_rva_to_ofs(&pe_shared->sections, *(enamet+i), &en)) goto pe_read_error;
		file->seek(en);
		char *s=getstrz(file);

		elinep=eline;
		elinep+=sprintf(elinep, "%04lx %08lx ", o, f);
		mktag_ref(elinep, o, tag_coloridx_sel_unselected, s, f);

		ht_pe_export_function *efd=new ht_pe_export_function(f, o, s);
		pe_shared->exports.funcs.add(efd);
		delete s;
		m->add_mask(eline);
	}

	if (!edir.function_count) m->add_mask(_SEL("ffffffff", "17")"<no exported functions>");

	v->insertsub(m);

	g->insert(head);
	g->insert(v);

	stop_timer(h0);
	LOG("%s: PE: it took %d ticks (%d msec) to read the exports", filename, get_timer_tick(h0), get_timer_msec(h0));
	delete_timer(h0);

	if (efunct) delete efunct;
	if (enamet) delete enamet;
	if (eordt) delete eordt;
	return g;
pe_read_error:
	if (efunct) delete efunct;
	if (enamet) delete enamet;
	if (eordt) delete eordt;
	delete m;
	delete v;
/* FIXME: return of zero not yet supported...(inits must be successful ;-) */
	assert(0);
	return 0;
}

viewer_int htpeexports_int = {
	htpeexports_detect,
	htpeexports_init
};

/*
 *	CLASS ht_pe_export_viewer
 */

ht_pe_export_viewer::ht_pe_export_viewer(bounds *b, char *desc, htstream *file, int caps)
: ht_uviewer(b, desc, file, caps)
{
}

ht_pe_export_viewer::~ht_pe_export_viewer()
{
}

void ht_pe_export_viewer::ref(dword id)
{
	ht_group *g=group->group;
	ht_viewer *v=(ht_viewer*)g->getbydesc(DESC_PE_IMAGE);
	if (v) {
		v->jump(id);
		group->group->focus(v);
	}
}

/*
 *	class ht_pe_export_function
 */

ht_pe_export_function::ht_pe_export_function()
{
	name=0;
	ordinal=0;
	function_address=0;
}

ht_pe_export_function::ht_pe_export_function(int function_address, int ordinal)
{
	this->ordinal=ordinal;
	this->function_address=function_address;
	byname=0;
}

ht_pe_export_function::ht_pe_export_function(int function_address, int ordinal, char *name)
{
	this->ordinal=ordinal;
	this->name=htstrdup(name);
	this->function_address=function_address;
	byname=1;
}

ht_pe_export_function::~ht_pe_export_function()
{
	if ((byname) && (name)) delete name;
}
