/* 
 *	The HT Editor
 *	htsearch.cc
 *
 *	Copyright (C) 1999,2000 Stefan Weyergraf (stefan@weyergraf.de)
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License version 2 as
 *	published by the Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "htsearch.h"

#define SEARCH_BUF_SIZE 2048

int bufequal(byte *buf1, byte *buf2, int len)
{
	for (int i=0; i<len; i++) {
		if (buf1[i]!=buf2[i]) return 0;
	}
	return 1;
}

int search_str(htstream *file, dword start, dword end, dword pattern_len, byte *pattern, dword *result_ofs)
{
	byte buf[SEARCH_BUF_SIZE];
	byte *bptr;
	dword b;
	dword o=start;
	int end_reached=0;
	if (pattern_len>=4) {
/* optimized for patterns of length >= 4 (dword size) */
		dword pdword=*((dword*)pattern);
		file->seek(o);
		b=SEARCH_BUF_SIZE;
		if (b+o>end) b=end-o;
		b=file->read(buf, b);
		while (1) {
			bptr=buf;
			while (bptr-buf+pattern_len<=b) {
				if (pdword==*((dword*)bptr)) {
					if (bufequal(pattern+4, bptr+4, pattern_len-4)) {
						*result_ofs=o+bptr-buf;
						return 1;
					}
				}
				bptr++;
			}
			o+=b-pattern_len;
			if (end_reached) break;
			file->seek(o);
			b=SEARCH_BUF_SIZE;
			if (b+o>end) {
				b=end-o;
				end_reached=1;
			}
			b=file->read(buf, b);
		}
	} else {
/* unoptimized */
		dword pbyte=*pattern;
		file->seek(o);
		b=SEARCH_BUF_SIZE;
		if (b+o>end) b=end-o;
		b=file->read(buf, b);
		while (1) {
			bptr=buf;
			while (bptr-buf+pattern_len<=b) {
				if (pbyte==*bptr) {
					if (bufequal(pattern, bptr, pattern_len)) {
						*result_ofs=o+bptr-buf;
						return 1;
					}
				}
				bptr++;
			}
			o+=b-pattern_len;
			if (end_reached) break;
			file->seek(o);
			b=SEARCH_BUF_SIZE;
			if (b+o>end) {
				b=end-o;
				end_reached=1;
			}
			b=file->read(buf, b);
		}
	}
	return 0;
}

