/* 
 *	The HT Editor
 *   htcurses.h (DJGPP implementation)
 *
 *	Copyright (C) 1999,2000 Stefan Weyergraf (stefan@weyergraf.de)
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License version 2 as
 *	published by the Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __HTCURSES_H__
#define __HTCURSES_H__

#define C_BLACK 0
#define C_BLUE 1
#define C_GREEN 2
#define C_CYAN 3
#define C_RED 4
#define C_MAGENTA 5
#define C_YELLOW 6
#define C_WHITE 7

#define C_LIGHT(c) (c+8)

#define C_FG(c) ((c) & 15)
#define C_BG(c) (((c)>>4) & 15)

#define COLOR(fg, bg) ((bg<<4)|fg)

#define CHAR_LINEV (unsigned char)'\xb3'
#define CHAR_LINEH (unsigned char)'\xc4'
#define CHAR_LINEH_BU (unsigned char)'\xdf'
#define CHAR_LINEH_BL (unsigned char)'\xdc'
#define CHAR_BORDERTL (unsigned char)'\xc3'
#define CHAR_BORDERTR (unsigned char)'\xb4'
#define CHAR_BORDERTU (unsigned char)'\xc2'
#define CHAR_BORDERTD (unsigned char)'\xc1'
#define CHAR_CORNERUL (unsigned char)'\xda'
#define CHAR_CORNERLL (unsigned char)'\xc0'
#define CHAR_CORNERUR (unsigned char)'\xbf'
#define CHAR_CORNERLR (unsigned char)'\xd9'
#define CHAR_FILLED_L (unsigned char)'\xb0'	/* FILLED, low density */
#define CHAR_FILLED_M (unsigned char)'\xb1'	/* FILLED, medium density */
#define CHAR_FILLED_H (unsigned char)'\xb2'  /* FILLED, high density */
#define CHAR_FILLED_F (unsigned char)'\xdb'  /* FILLED, filled entirely */
#define CHAR_FILLED_HU (unsigned char)'\xdf' /* FILLED, upper half - filled entirely */
#define CHAR_FILLED_HL (unsigned char)'\xdc' /* FILLED, lower half - filled entirely */
#define CHAR_QUAD_SMALL (unsigned char)'\xfe'
#define CHAR_ARROW_UP (unsigned char)'\x18'
#define CHAR_ARROW_DOWN (unsigned char)'\x19'
#define CHAR_ARROWBIG_UP (unsigned char)'\x1e'
#define CHAR_ARROWBIG_DOWN (unsigned char)'\x1f'

#include "common.h"

#include <stdarg.h>

struct drawbufch {
	int ch;
	int c;
};

/*
 *	CLASS genericdrawbuf
 */

class genericdrawbuf {
public:
	bounds size;

	genericdrawbuf();
	virtual ~genericdrawbuf();
	virtual void fill(int x, int y, int w, int h, int c, int ch);
		   int printf(int x, int y, int c, char *format, ...);
	virtual int print(int x, int y, int c, char *text);
	virtual void printchar(int x, int y, int c, int ch);
	virtual int lprint(int x, int y, int c, int l, char *text);
	virtual void setbounds(bounds *b);
};

/*
 *	CLASS drawbuf
 */

class drawbuf: public genericdrawbuf {
public:
	drawbufch *buf;

	drawbuf(bounds *b);
	~drawbuf();
	void fill(int x, int y, int w, int h, int c, int ch);
	int print(int x, int y, int c, char *text);
	void printchar(int x, int y, int c, int ch);
	int lprint(int x, int y, int c, int l, char *text);
	void setbounds(bounds *b);
};

class screendrawbuf: public genericdrawbuf {
protected:
	unsigned short *buf;
	int screensel;
	int cursorx, cursory;
public:
	screendrawbuf();
	~screendrawbuf();
	void fill(int x, int y, int w, int h, int c, int ch);
	int print(int x, int y, int c, char *text);
	void printchar(int x, int y, int c, int ch);
	int lprint(int x, int y, int c, int l, char *text);
	void setbounds(bounds *b);
/* new */
	void drawbuffer(drawbuf *buf, int x, int y);
	void	show();
	void getcursor(int *x, int *y);
	void hidecursor();
	void showcursor();
	void setcursor(int x, int y);
};

/*
 *	misc
 */

#endif /* !__HTCURSES_H__ */
